@@ -59,9 +59,9 @@ where
 
 /// Returns the result of the locator at the walker
 /// Returns None if the walker is at an empty position
-pub fn walker_locate<W, D: Data, L>(walker: &mut W, locator: &L) -> Option<LocResult>
+pub fn query_locator<W, D: Data, L>(walker: &mut W, locator: &L) -> Option<LocResult>
 where
-    W: crate::trees::SomeWalker<D>,
+    W: crate::trees::SomeWalker<D> + ?Sized,
     L: Locator<D>,
 {
     if let Some(value) = walker.value() {
@@ -73,8 +73,25 @@ where
     }
 }
 
-// TODO: Splitter. locators that can't `Accept`. used for splitting trees
-// and for insertions.
+/// Returns the result of the locator at the walker
+/// Returns None if the walker is at an empty position
+pub fn clone_locate<D: Data, L>(
+    current_action: D::Action,
+    left: D::Summary,
+    value: &D::Value,
+    right: D::Summary,
+    locator: &L,
+) -> LocResult
+where
+    L: Locator<D>,
+    D::Value: Clone,
+{
+    if !current_action.to_reverse() {
+        locator.locate(left, &current_action.act(value.clone()), right)
+    } else {
+        locator.locate(right, &current_action.act(value.clone()), left)
+    }
+}
 
 /// Locator instance for `usize` representing a single index.
 impl<D: Data> Locator<D> for usize
@@ -87,7 +104,7 @@ where
 
         if s > *self {
             GoLeft
-        } else if s + D::to_summary(node).size() <= *self {
+        } else if s + node.to_summary().size() <= *self {
             GoRight
         } else {
             Accept
@@ -120,7 +137,7 @@ where
 
         if s >= self.end {
             GoLeft
-        } else if s + D::to_summary(node).size() <= self.start {
+        } else if s + node.to_summary().size() <= self.start {
             GoRight
         } else {
             Accept
@@ -139,7 +156,7 @@ where
 
         if s >= self.end {
             GoLeft
-        } else if s + D::to_summary(node).size() <= self.start {
+        } else if s + node.to_summary().size() <= self.start {
             GoRight
         } else {
             Accept
@@ -159,7 +176,7 @@ where
 
         if s > *self.end() {
             GoLeft
-        } else if s + D::to_summary(node).size() <= *self.start() {
+        } else if s + node.to_summary().size() <= *self.start() {
             GoRight
         } else {
             Accept
@@ -179,7 +196,7 @@ where
 
         if s > *self.end() {
             GoLeft
-        } else if s + D::to_summary(node).size() <= *self.start() {
+        } else if s + node.to_summary().size() <= *self.start() {
             GoRight
         } else {
             Accept
@@ -196,7 +213,7 @@ where
         // find the index of the current node
         let s = left.size();
 
-        if s + D::to_summary(node).size() <= self.start {
+        if s + node.to_summary().size() <= self.start {
             GoRight
         } else {
             Accept
@@ -213,7 +230,7 @@ where
         // find the index of the current node
         let s = left.size();
 
-        if s + D::to_summary(node).size() <= self.start {
+        if s + node.to_summary().size() <= self.start {
             GoRight
         } else {
             Accept
@@ -301,9 +318,9 @@ pub struct ByKey<T>(pub T);
 
 /// Can't be an instance for `ByKey<D::Value::Key>` directly, because the `Key` might itself
 /// be a range type, and so it would conflict with the other implementations.
-impl<'a, D: Data> Locator<D> for ByKey<(&<D::Value as Keyed>::Key,)>
+impl<'a, D: Data, Key: Ord> Locator<D> for ByKey<(&Key,)>
 where
-    D::Value: Keyed,
+    D::Value: Keyed<Key>,
 {
     fn locate(&self, _left: D::Summary, node: &D::Value, _right: D::Summary) -> LocResult {
         match node.get_key().cmp(self.0 .0) {
@@ -322,9 +339,9 @@ impl<D: Data> Locator<D> for ByKey<std::ops::RangeFull> {
 }
 
 /// Locator instance for [`ByKey`]`<std::ops::Range<D::Value::Key>>` representing searching by a key.
-impl<'a, D: Data> Locator<D> for ByKey<std::ops::Range<&'a <D::Value as Keyed>::Key>>
+impl<D: Data, Key: Ord> Locator<D> for ByKey<std::ops::Range<&Key>>
 where
-    D::Value: Keyed,
+    D::Value: Keyed<Key>,
 {
     fn locate(&self, _left: D::Summary, node: &D::Value, _right: D::Summary) -> LocResult {
         // find the index of the current node
@@ -361,9 +378,9 @@ impl<D: Data> Locator<D> for &ByKey<std::ops::Range<<D::Value as Keyed>::Key>> w
 
 /// Locator instance for [`ByKey`]`<std::ops::RangeInclusive<D::Value::Key>>` representing searching by a key.
 /// Do not use with ranges that have been iterated on to exhaustion.
-impl<'a, D: Data> Locator<D> for ByKey<std::ops::RangeInclusive<&'a <D::Value as Keyed>::Key>>
+impl<D: Data, Key: Ord> Locator<D> for ByKey<std::ops::RangeInclusive<&Key>>
 where
-    D::Value: Keyed,
+    D::Value: Keyed<Key>,
 {
     fn locate(&self, _left: D::Summary, node: &D::Value, _right: D::Summary) -> LocResult {
         // find the index of the current node
@@ -400,9 +417,9 @@ impl<D: Data> Locator<D> for &ByKey<std::ops::RangeInclusive<<D::Value as Keyed>
 */
 
 /// Locator instance for [`ByKey`]`<`[`std::ops::RangeFrom`]`<D::Value::Key>>` representing an index range.
-impl<'a, D: Data> Locator<D> for ByKey<std::ops::RangeFrom<&'a <D::Value as Keyed>::Key>>
+impl<D: Data, Key: Ord> Locator<D> for ByKey<std::ops::RangeFrom<&Key>>
 where
-    D::Value: Keyed,
+    D::Value: Keyed<Key>,
 {
     fn locate(&self, _left: D::Summary, node: &D::Value, _right: D::Summary) -> LocResult {
         // find the index of the current node
@@ -416,9 +433,9 @@ where
 }
 
 /// Locator instance for [`ByKey`]`<std::ops::RangeTo<D::Value::Key>>` representing searching by a key.
-impl<'a, D: Data> Locator<D> for ByKey<std::ops::RangeTo<&'a <D::Value as Keyed>::Key>>
+impl<D: Data, Key: Ord> Locator<D> for ByKey<std::ops::RangeTo<&Key>>
 where
-    D::Value: Keyed,
+    D::Value: Keyed<Key>,
 {
     fn locate(&self, _left: D::Summary, node: &D::Value, _right: D::Summary) -> LocResult {
         // find the index of the current node
@@ -432,9 +449,9 @@ where
 }
 
 /// Locator instance for [`ByKey`]`<std::ops::RangeToInclusive<D::Value::Key>>` representing searching by a key.
-impl<'a, D: Data> Locator<D> for ByKey<std::ops::RangeToInclusive<&'a <D::Value as Keyed>::Key>>
+impl<D: Data, Key: Ord> Locator<D> for ByKey<std::ops::RangeToInclusive<&Key>>
 where
-    D::Value: Keyed,
+    D::Value: Keyed<Key>,
 {
     fn locate(&self, _left: D::Summary, node: &D::Value, _right: D::Summary) -> LocResult {
         // find the index of the current node
@@ -447,9 +464,6 @@ where
     }
 }
 
-// TODO: finish all the range types
-// TODO: switch to ranges of references to keys
-
 /// A Wrapper for other locators what will find exactly the left edge
 /// of the previous locator. So, this is always a splitting locator.
 #[derive(Clone, Copy, PartialEq, Eq, Hash, Debug)]
