@@ -132,11 +132,19 @@ impl<D: Data> Default for AVLTree<D> {
 }
 
 impl<D: Data> SomeTree<D> for AVLTree<D> {
+    fn segment_summary_imm<L>(&self, locator: L) -> D::Summary
+    where
+        L: crate::Locator<D>,
+        D::Value: Clone,
+    {
+        segment_algorithms::segment_summary_imm(&self.tree, locator)
+    }
+
     fn segment_summary<L>(&mut self, locator: L) -> D::Summary
     where
         L: crate::Locator<D>,
     {
-        methods::segment_summary(self, locator)
+        segment_algorithms::segment_summary(self, locator)
     }
 
     fn act_segment<L>(&mut self, action: D::Action, locator: L)
@@ -144,7 +152,7 @@ impl<D: Data> SomeTree<D> for AVLTree<D> {
         L: crate::Locator<D>,
     {
         if !action.to_reverse() {
-            methods::act_segment(self, action, locator)
+            segment_algorithms::act_segment(self, action, locator)
         } else {
             // split out the middle
             let mut mid: AVLTree<D> = self
@@ -159,7 +167,7 @@ impl<D: Data> SomeTree<D> for AVLTree<D> {
                     Default::default(),
                 ),
             };
-            methods::search_subtree(&mut walker2, locators::RightEdgeOf(locator));
+            walker2.search_subtree(locators::RightEdgeOf(locator));
             let right = walker2.split_right().unwrap();
             drop(walker2);
 
@@ -211,53 +219,16 @@ impl<'a, D: Data> SplittableTreeRef<D> for &'a mut AVLTree<D> {
     type SplittableWalker = AVLWalker<'a, D>;
 }
 
-impl<D: Data> SomeEntry<D> for AVLTree<D> {
-    fn with_value<F, R>(&mut self, f: F) -> Option<R>
-    where
-        F: FnOnce(&mut D::Value) -> R,
-    {
-        self.tree.with_value(f)
-    }
-
-    fn node_summary(&self) -> D::Summary {
-        self.tree.node_summary()
-    }
-
-    fn subtree_summary(&self) -> D::Summary {
-        self.tree.subtree_summary()
-    }
-
-    fn left_subtree_summary(&self) -> Option<D::Summary> {
-        self.tree.left_subtree_summary()
-    }
-
-    fn right_subtree_summary(&self) -> Option<D::Summary> {
-        self.tree.right_subtree_summary()
-    }
-
-    fn act_subtree(&mut self, action: D::Action) {
-        self.tree.act_subtree(action);
-    }
-
-    fn act_node(&mut self, action: D::Action) -> Option<()> {
-        self.tree.act_node(action)
-    }
-
-    fn act_left_subtree(&mut self, action: D::Action) -> Option<()> {
-        self.tree.act_left_subtree(action)
-    }
-
-    fn act_right_subtree(&mut self, action: D::Action) -> Option<()> {
-        self.tree.act_right_subtree(action)
-    }
-
-    fn assert_correctness_locally(&self)
-    where
-        D::Summary: Eq,
-    {
-        if let Some(node) = self.tree.node() {
-            Self::assert_ranks_locally_internal(node);
-            node.assert_correctness_locally();
+derive_SomeEntry! {tree, T,
+    impl<D: Data> SomeEntry<D> for AVLTree<D> {
+        fn assert_correctness_locally(&self)
+        where
+            D::Summary: Eq,
+        {
+            if let Some(node) = self.tree.node() {
+                Self::assert_ranks_locally_internal(node);
+                node.assert_correctness_locally();
+            }
         }
     }
 }
@@ -274,7 +245,6 @@ impl<D: Data> std::iter::FromIterator<D::Value> for AVLTree<D> {
         for val in iter.into_iter() {
             // note: this relies on the assumption, that after we insert a node, the new position of the locator
             // will be an ancestor of the location where the value was inserted.
-            // TODO: check.
             while walker.go_right().is_ok() {}
             walker.insert(val);
         }
@@ -303,86 +273,27 @@ impl<'a, D: Data> std::ops::Drop for AVLWalker<'a, D> {
     }
 }
 
-impl<'a, D: Data> SomeWalker<D> for AVLWalker<'a, D> {
-    fn go_left(&mut self) -> Result<(), ()> {
-        self.walker.go_left()
-    }
-
-    fn go_right(&mut self) -> Result<(), ()> {
-        self.walker.go_right()
-    }
-
-    fn go_up(&mut self) -> Result<Side, ()> {
-        let res = self.walker.go_up()?;
-        let changed = self.inner_mut().rebuild_ranks();
-        assert!(!changed); // it shouldn't have changed without being rebalanced already
-        Ok(res)
-    }
-
-    fn depth(&self) -> usize {
-        self.walker.depth()
-    }
-
-    fn far_left_summary(&self) -> D::Summary {
-        self.walker.far_left_summary()
-    }
-
-    fn far_right_summary(&self) -> D::Summary {
-        self.walker.far_left_summary()
-    }
-
-    fn value(&self) -> Option<&D::Value> {
-        self.walker.value()
+derive_SomeWalker! {walker,
+    impl<'a, D: Data> SomeWalker<D> for AVLWalker<'a, D> {
+        fn go_up(&mut self) -> Result<Side, ()> {
+            let res = self.walker.go_up()?;
+            let changed = self.inner_mut().rebuild_ranks();
+            assert!(!changed); // it shouldn't have changed without being rebalanced already
+            Ok(res)
+        }
     }
 }
 
-impl<'a, D: Data> SomeEntry<D> for AVLWalker<'a, D> {
-    fn with_value<F, R>(&mut self, f: F) -> Option<R>
-    where
-        F: FnOnce(&mut D::Value) -> R,
-    {
-        self.walker.with_value(f)
-    }
-
-    fn node_summary(&self) -> D::Summary {
-        self.walker.node_summary()
-    }
-
-    fn subtree_summary(&self) -> D::Summary {
-        self.walker.subtree_summary()
-    }
-
-    fn left_subtree_summary(&self) -> Option<D::Summary> {
-        self.walker.left_subtree_summary()
-    }
-
-    fn right_subtree_summary(&self) -> Option<D::Summary> {
-        self.walker.right_subtree_summary()
-    }
-
-    fn act_subtree(&mut self, action: D::Action) {
-        self.walker.act_subtree(action);
-    }
-
-    fn act_node(&mut self, action: D::Action) -> Option<()> {
-        self.walker.act_node(action)
-    }
-
-    fn act_left_subtree(&mut self, action: D::Action) -> Option<()> {
-        self.walker.act_left_subtree(action)
-    }
-
-    fn act_right_subtree(&mut self, action: D::Action) -> Option<()> {
-        self.walker.act_right_subtree(action)
-    }
-
-    fn assert_correctness_locally(&self)
-    where
-        D::Summary: Eq,
-    {
-        self.walker.assert_correctness_locally();
-        if let Some(node) = self.walker.node() {
-            AVLTree::assert_ranks_locally_internal(node);
+derive_SomeEntry! {walker, T,
+    impl<'a, D: Data> SomeEntry<D> for AVLWalker<'a, D> {
+        fn assert_correctness_locally(&self)
+        where
+            D::Summary: Eq,
+        {
+            self.walker.assert_correctness_locally();
+            if let Some(node) = self.walker.node() {
+                AVLTree::assert_ranks_locally_internal(node);
+            }
         }
     }
 }
@@ -469,7 +380,7 @@ impl<'a, D: Data> AVLWalker<'a, D> {
                     } else {
                         // left.rank() = 1, left right case
                         self.go_left().unwrap();
-                        self.rot_left().unwrap(); // TODO
+                        self.rot_left().unwrap();
                         let res = self.rot_up();
                         assert!(res == Ok(Side::Left));
                     }
@@ -510,10 +421,11 @@ impl<'a, D: Data> AVLWalker<'a, D> {
         }
     }
 
-    // TODO: specify where the walker will be.
     /// Deletes a node and returns it with the box.
+    /// The walker reorganizes the current subtree in order to delete the current node,
+    /// and then rebalances. During rebalancing it may only go up the tree.
     fn delete_boxed(&mut self) -> Option<Box<BasicNode<D, T>>> {
-        // the delete implementation is copied from `BasicTree`,
+        // the delete implementation is modified from `BasicTree`,
         // in order for rebalancing to be done properly.
         let mut node = self.walker.take_subtree().into_node_boxed()?;
         if node.right.is_empty() {
@@ -538,7 +450,7 @@ impl<'a, D: Data> AVLWalker<'a, D> {
             node.right = BasicTree::Empty;
             boxed_replacement_node.rebuild();
             self.walker
-                .put_subtree(BasicTree::Root(boxed_replacement_node))
+                .put_subtree(BasicTree::from_boxed_node(boxed_replacement_node))
                 .unwrap();
             self.rebalance(); // rebalance here
         }
@@ -558,7 +470,9 @@ impl<'a, D: Data> ModifiableWalker<D> for AVLWalker<'a, D> {
         Some(())
     }
 
-    // TODO: specify where the walker will be.
+    
+    /// The walker reorganizes the current subtree in order to delete the current node,
+    /// and then rebalances. During rebalancing it may only go up the tree.
     fn delete(&mut self) -> Option<D::Value> {
         Some(self.delete_boxed()?.node_value)
     }
@@ -574,10 +488,8 @@ impl<'a, D: Data> SplittableWalker<D> for AVLWalker<'a, D> {
     /// The walker will be at the root after this operation, if it succeeds.
     ///
     ///```
-    /// use grove::trees::*;
-    /// use grove::avl::*;
+    /// use grove::{SomeTree, avl::AVLTree};
     /// use grove::example_data::StdNum;
-    /// use grove::methods::*;
     ///
     /// let mut tree: AVLTree<StdNum> = (17..88).collect();
     /// let mut tree2 = tree.slice(7..7).split_right().unwrap();
@@ -624,10 +536,8 @@ impl<'a, D: Data> SplittableWalker<D> for AVLWalker<'a, D> {
     /// The walker will be at the root after this operation, if it succeeds.
     ///
     ///```
-    /// use grove::trees::*;
-    /// use grove::avl::*;
+    /// use grove::{SomeTree, avl::AVLTree};
     /// use grove::example_data::StdNum;
-    /// use grove::methods::*;
     ///
     /// let mut tree: AVLTree<StdNum> = (17..88).collect();
     /// let mut tree2 = tree.slice(7..7).split_left().unwrap();
@@ -647,8 +557,7 @@ impl<D: Data> AVLTree<D> {
     /// Concatenates the trees together, in place, with a given value for the middle.
     /// Complexity: `O(log n)`. More precisely, `O(dr)` where `dr` is the difference of ranks between the two trees.
     ///```
-    /// use grove::trees::*;
-    /// use grove::avl::*;
+    /// use grove::{SomeTree, avl::AVLTree};
     /// use grove::example_data::StdNum;
     ///
     /// let mut tree: AVLTree<StdNum> = (17..=89).collect();
@@ -681,15 +590,14 @@ impl<D: Data> AVLTree<D> {
         mid.left = walker.walker.take_subtree();
         mid.right = right.tree;
         mid.rebuild();
-        walker.walker.put_subtree(BasicTree::Root(mid)).unwrap();
+        walker.walker.put_subtree(BasicTree::from_boxed_node(mid)).unwrap();
         walker.rebalance();
     }
 
     /// Concatenates the trees together, in place, with a given value for the middle.
     /// Complexity: `O(log n)`. More precisely, `O(dr)` where `dr` is the difference of ranks between the two trees.
     ///```
-    /// use grove::trees::*;
-    /// use grove::avl::*;
+    /// use grove::{SomeTree, avl::AVLTree};
     /// use grove::example_data::StdNum;
     ///
     /// let tree1: AVLTree<StdNum> = (17..=89).collect();
@@ -722,7 +630,7 @@ impl<D: Data> AVLTree<D> {
         mid.right = walker.walker.take_subtree();
         mid.left = left.tree;
         mid.rebuild();
-        walker.walker.put_subtree(BasicTree::Root(mid)).unwrap();
+        walker.walker.put_subtree(BasicTree::from_boxed_node(mid)).unwrap();
         walker.rebalance();
     }
 }
@@ -731,8 +639,7 @@ impl<D: Data> ConcatenableTree<D> for AVLTree<D> {
     /// Concatenates the trees together, in place.
     /// Complexity: `O(log n)`.
     ///```
-    /// use grove::trees::*;
-    /// use grove::avl::*;
+    /// use grove::{SomeTree, ConcatenableTree, avl::AVLTree};
     /// use grove::example_data::StdNum;
     ///
     /// let mut tree: AVLTree<StdNum> = (17..=89).collect();
@@ -744,9 +651,9 @@ impl<D: Data> ConcatenableTree<D> for AVLTree<D> {
     ///```
     fn concatenate_right(&mut self, mut right: Self) {
         if !right.is_empty() {
-            let mut walker = methods::search(&mut right, locators::LeftEdgeOf(..));
+            let mut walker = right.search(locators::LeftEdgeOf(..));
             walker.go_up().unwrap();
-            let mid = walker.delete_boxed().unwrap(); // TODO: deallocated node only to reallocate it later. fix.
+            let mid = walker.delete_boxed().unwrap();
             drop(walker);
             self.concatenate_boxed_middle_right(mid, right);
         }
@@ -756,8 +663,7 @@ impl<D: Data> ConcatenableTree<D> for AVLTree<D> {
 /// Concatenates the trees together, in place, with a given value for the middle.
 /// Complexity: `O(log n)`. More precisely, `O(dr)` where `dr` is the difference of ranks between the two trees.
 ///```
-/// use grove::trees::*;
-/// use grove::avl::*;
+/// use grove::{SomeTree, avl::AVLTree, avl::concatenate_with_middle};
 /// use grove::example_data::StdNum;
 ///
 /// let tree1: AVLTree<StdNum> = (17..=89).collect();
