@@ -110,7 +110,8 @@ impl<D: Data> SplayTree<D> {
         };
 
         let right_edge = locators::RightEdgeOf(locator);
-        methods::search_walker(&mut walker2, right_edge);
+        walker2.go_to_root();
+        walker2.search_subtree(right_edge);
         let b2 = walker2.next_filled().is_ok();
         walker2.splay();
         drop(walker2);
@@ -161,9 +162,7 @@ impl<'a, D: Data> SplayWalker<'a, D> {
 
     // using this function can really mess up the structure
     // use wisely
-    // this function shouldn't really be public
-    // TODO: should this function exist?
-    pub(super) fn inner_mut(&mut self) -> &mut BasicTree<D> {
+    fn inner_mut(&mut self) -> &mut BasicTree<D> {
         self.walker.inner_mut()
     }
 
@@ -278,74 +277,6 @@ impl<'a, D: Data> SplayWalker<'a, D> {
             self.splay_step_depth(depth);
         }
     }
-
-    /// Finds the previous filled node.
-    /// If there isn't any, moves to root and return Err(()).
-    pub fn previous_filled(&mut self) -> Result<(), ()> {
-        match self.walker.node() {
-            None => {}
-            Some(node) => {
-                if !node.left.is_empty() {
-                    // the previous node is in this node's left subtree case
-                    self.go_left().unwrap();
-                    while self.go_right().is_ok() {}
-                    let r = self.go_up();
-                    assert_eq!(r, Ok(Side::Right));
-                    return Ok(());
-                }
-            }
-        }
-
-        // the next filled node is this node's first left ancestor
-        let count = match self.walker.steps_until_sided_ancestor(Side::Right) {
-            None => {
-                self.splay();
-                return Err(());
-            }
-            Some(count) => count,
-        };
-
-        let depth = self.depth();
-        // splay to just below the previous filled value
-        self.splay_to_depth(depth - count + 1);
-        let r = self.go_up();
-        assert_eq!(r, Ok(Side::Right));
-        Ok(())
-    }
-
-    /// Finds the next filled node.
-    /// If there isn't any, moves to root and return Err(()).
-    pub fn next_filled(&mut self) -> Result<(), ()> {
-        match self.walker.node() {
-            None => {}
-            Some(node) => {
-                if !node.right.is_empty() {
-                    // the previous node is in this node's right subtree case
-                    self.go_right().unwrap();
-                    while self.go_left().is_ok() {}
-                    let r = self.go_up();
-                    assert_eq!(r, Ok(Side::Left));
-                    return Ok(());
-                }
-            }
-        }
-        // return methods::next_filled(self);
-        // the next filled node is this node's first right ancestor
-        let count = match self.walker.steps_until_sided_ancestor(Side::Left) {
-            None => {
-                self.splay();
-                return Err(());
-            }
-            Some(count) => count,
-        };
-
-        let depth = self.depth();
-        // splay to just below the previous filled value
-        self.splay_to_depth(depth - count + 1);
-        let r = self.go_up();
-        assert_eq!(r, Ok(Side::Left));
-        Ok(())
-    }
 }
 
 impl<'a, D: Data> Drop for SplayWalker<'a, D> {
@@ -355,6 +286,27 @@ impl<'a, D: Data> Drop for SplayWalker<'a, D> {
 }
 
 impl<D: Data> SomeTree<D> for SplayTree<D> {
+    /// Note: calling this is inefficient
+    /// and panicks if debug assertions are on.
+    ///
+    /// This is because splay trees rely on changing the tree's structure to ensure
+    /// its complexity properties.
+    /// Using this might take `O(n)` time per operation in the worst case.
+    fn segment_summary_imm<L>(&self, locator: L) -> D::Summary
+    where
+        L: locators::Locator<D>,
+        D::Value: Clone,
+    {
+        if cfg!(debug_assertions) {
+            panic!(".segment_summary_imm() method is inefficient for splay trees")
+        } else {
+            self.tree.segment_summary_imm(locator)
+        }
+    }
+
+    /// This is the same as `self.segment_summary_unclonable(locator)`
+    /// because splay trees rely on changing the tree's structure to ensure
+    /// its complexity properties.
     fn segment_summary<L>(&mut self, locator: L) -> D::Summary
     where
         L: locators::Locator<D>,
@@ -388,51 +340,14 @@ impl<D: Data> SomeTree<D> for SplayTree<D> {
     }
 }
 
-impl<D: Data> SomeEntry<D> for SplayTree<D> {
-    fn node_summary(&self) -> D::Summary {
-        self.tree.node_summary()
-    }
-
-    fn subtree_summary(&self) -> D::Summary {
-        self.tree.subtree_summary()
-    }
-
-    fn left_subtree_summary(&self) -> Option<D::Summary> {
-        self.tree.left_subtree_summary()
-    }
-
-    fn right_subtree_summary(&self) -> Option<D::Summary> {
-        self.tree.right_subtree_summary()
-    }
-
-    fn with_value<F, R>(&mut self, f: F) -> Option<R>
-    where
-        F: FnOnce(&mut D::Value) -> R,
-    {
-        self.tree.with_value(f)
-    }
-
-    fn act_subtree(&mut self, action: D::Action) {
-        self.tree.act_subtree(action);
-    }
-
-    fn act_node(&mut self, action: D::Action) -> Option<()> {
-        self.tree.act_node(action)
-    }
-
-    fn act_left_subtree(&mut self, action: D::Action) -> Option<()> {
-        self.tree.act_left_subtree(action)
-    }
-
-    fn act_right_subtree(&mut self, action: D::Action) -> Option<()> {
-        self.tree.act_right_subtree(action)
-    }
-
-    fn assert_correctness_locally(&self)
-    where
-        D::Summary: Eq,
-    {
-        self.tree.assert_correctness_locally();
+derive_SomeEntry! {tree, (),
+    impl<D: Data> SomeEntry<D> for SplayTree<D> {
+        fn assert_correctness_locally(&self)
+        where
+            D::Summary: Eq,
+        {
+            self.tree.assert_correctness_locally();
+        }
     }
 }
 
@@ -465,92 +380,102 @@ impl<D: Data> IntoIterator for SplayTree<D> {
     }
 }
 
-impl<'a, D: Data> SomeWalker<D> for SplayWalker<'a, D> {
-    fn go_left(&mut self) -> Result<(), ()> {
-        self.walker.go_left()
-    }
+derive_SomeWalker! {walker,
+    impl<'a, D: Data> SomeWalker<D> for SplayWalker<'a, D> {
+        /// If successful, returns whether or not the previous current value was the left son.
+        /// If already at the root of the tree, returns `Err(())`.
+        /// You shouldn't use this method too much, or you might lose the
+        /// SplayTree's complexity properties - see documentation aboud splay tree.
+        fn go_up(&mut self) -> Result<Side, ()> {
+            self.walker.go_up()
+        }
 
-    fn go_right(&mut self) -> Result<(), ()> {
-        self.walker.go_right()
-    }
+        // overrides the default implementations for these methods:
 
-    /// If successful, returns whether or not the previous current value was the left son.
-    /// If already at the root of the tree, returns `Err(())`.
-    /// You shouldn't use this method too much, or you might lose the
-    /// SplayTree's complexity properties - see documentation aboud splay tree.
-    fn go_up(&mut self) -> Result<Side, ()> {
-        self.walker.go_up()
-    }
+        /// Finds the previous filled node.
+        /// If there isn't any, moves to root and return Err(()).
+        ///
+        /// Restructures the tree in order to satisfy the splay tree's complexity properties.
+        /// Complexity: amortized `O(log n)` time.
+        fn previous_filled(&mut self) -> Result<(), ()> {
+            match self.walker.node() {
+                None => {}
+                Some(node) => {
+                    if !node.left.is_empty() {
+                        // the previous node is in this node's left subtree case
+                        self.go_left().unwrap();
+                        while self.go_right().is_ok() {}
+                        let r = self.go_up();
+                        assert_eq!(r, Ok(Side::Right));
+                        return Ok(());
+                    }
+                }
+            }
 
-    fn go_to_root(&mut self) {
-        self.splay();
-    }
+            // the next filled node is this node's first left ancestor
+            let count = match self.walker.steps_until_sided_ancestor(Side::Right) {
+                None => {
+                    self.splay();
+                    return Err(());
+                }
+                Some(count) => count,
+            };
 
-    fn depth(&self) -> usize {
-        self.walker.depth()
-    }
+            let depth = self.depth();
+            // splay to just below the previous filled value
+            self.splay_to_depth(depth - count + 1);
+            let r = self.go_up();
+            assert_eq!(r, Ok(Side::Right));
+            Ok(())
+        }
 
-    fn far_left_summary(&self) -> D::Summary {
-        self.walker.far_left_summary()
-    }
-    fn far_right_summary(&self) -> D::Summary {
-        self.walker.far_right_summary()
-    }
+        /// Finds the next filled node.
+        /// If there isn't any, moves to root and return Err(()).
+        ///
+        /// Restructures the tree in order to satisfy the splay tree's complexity properties.
+        /// Complexity: amortized `O(log n)` time.
+        fn next_filled(&mut self) -> Result<(), ()> {
+            match self.walker.node() {
+                None => {}
+                Some(node) => {
+                    if !node.right.is_empty() {
+                        // the previous node is in this node's right subtree case
+                        self.go_right().unwrap();
+                        while self.go_left().is_ok() {}
+                        let r = self.go_up();
+                        assert_eq!(r, Ok(Side::Left));
+                        return Ok(());
+                    }
+                }
+            }
+            // return methods::next_filled(self);
+            // the next filled node is this node's first right ancestor
+            let count = match self.walker.steps_until_sided_ancestor(Side::Left) {
+                None => {
+                    self.splay();
+                    return Err(());
+                }
+                Some(count) => count,
+            };
 
-    // fn inner(&self) -> &BasicTree<A> {
-    //     self.walker.inner()
-    // }
-
-    fn value(&self) -> Option<&D::Value> {
-        self.walker.value()
+            let depth = self.depth();
+            // splay to just below the previous filled value
+            self.splay_to_depth(depth - count + 1);
+            let r = self.go_up();
+            assert_eq!(r, Ok(Side::Left));
+            Ok(())
+        }
     }
 }
 
-impl<'a, D: Data> SomeEntry<D> for SplayWalker<'a, D> {
-    fn node_summary(&self) -> D::Summary {
-        self.walker.node_summary()
-    }
-
-    fn subtree_summary(&self) -> D::Summary {
-        self.walker.subtree_summary()
-    }
-
-    fn left_subtree_summary(&self) -> Option<D::Summary> {
-        self.walker.left_subtree_summary()
-    }
-
-    fn right_subtree_summary(&self) -> Option<D::Summary> {
-        self.walker.right_subtree_summary()
-    }
-
-    fn with_value<F, R>(&mut self, f: F) -> Option<R>
-    where
-        F: FnOnce(&mut D::Value) -> R,
-    {
-        self.walker.with_value(f)
-    }
-
-    fn act_subtree(&mut self, action: D::Action) {
-        self.walker.act_subtree(action);
-    }
-
-    fn act_node(&mut self, action: D::Action) -> Option<()> {
-        self.walker.act_node(action)
-    }
-
-    fn act_left_subtree(&mut self, action: D::Action) -> Option<()> {
-        self.walker.act_left_subtree(action)
-    }
-
-    fn act_right_subtree(&mut self, action: D::Action) -> Option<()> {
-        self.walker.act_right_subtree(action)
-    }
-
-    fn assert_correctness_locally(&self)
-    where
-        D::Summary: Eq,
-    {
-        self.walker.assert_correctness_locally();
+derive_SomeEntry! {walker, (),
+    impl<'a, D: Data> SomeEntry<D> for SplayWalker<'a, D> {
+        fn assert_correctness_locally(&self)
+        where
+            D::Summary: Eq,
+        {
+            self.walker.assert_correctness_locally();
+        }
     }
 }
 
@@ -589,7 +514,7 @@ impl<'a, D: Data> ModifiableWalker<D> for SplayWalker<'a, D> {
             boxed_replacement_node.right = node.right;
             boxed_replacement_node.rebuild();
             self.walker
-                .put_subtree(BasicTree::Root(boxed_replacement_node))
+                .put_subtree(BasicTree::from_boxed_node(boxed_replacement_node))
                 .unwrap();
         }
         Some(node.node_value)
@@ -597,12 +522,9 @@ impl<'a, D: Data> ModifiableWalker<D> for SplayWalker<'a, D> {
 }
 
 impl<D: Data> ConcatenableTree<D> for SplayTree<D> {
-    // `tree3 = union(tree1, tree2)`, not
-    // `tree1.concatenate(tree2)`.
     /// Concatenates the other tree into this tree.
     ///```
-    /// use grove::trees::*;
-    /// use grove::splay::*;
+    /// use grove::{SomeTree, ConcatenableTree, splay::SplayTree};
     /// use grove::example_data::StdNum;
     ///
     /// let tree1: SplayTree<StdNum> = (17..=89).collect();
@@ -649,10 +571,8 @@ impl<'a, D: Data> SplittableWalker<D> for SplayWalker<'a, D> {
     /// The walker will be at the root after this operation, if it succeeds.
     ///
     ///```
-    /// use grove::trees::*;
-    /// use grove::splay::*;
+    /// use grove::{SomeTree, SplittableTreeRef, splay::SplayTree};
     /// use grove::example_data::StdNum;
-    /// use grove::methods::*;
     ///
     /// let mut tree: SplayTree<StdNum> = (17..88).collect();
     /// let mut tree2 = tree.slice(7..7).split_right().unwrap();
