@@ -27,11 +27,19 @@ pub struct Treap<D: Data> {
 }
 
 impl<D: Data> SomeTree<D> for Treap<D> {
+    fn segment_summary_imm<L>(&self, locator: L) -> D::Summary
+    where
+        L: locators::Locator<D>,
+        D::Value: Clone,
+    {
+        segment_algorithms::segment_summary_imm(&self.tree, locator)
+    }
+
     fn segment_summary<L>(&mut self, locator: L) -> D::Summary
     where
         L: crate::Locator<D>,
     {
-        methods::segment_summary(self, locator)
+        segment_algorithms::segment_summary(self, locator)
     }
 
     fn act_segment<L>(&mut self, action: D::Action, locator: L)
@@ -39,7 +47,7 @@ impl<D: Data> SomeTree<D> for Treap<D> {
         L: crate::Locator<D>,
     {
         if !action.to_reverse() {
-            methods::act_segment(self, action, locator)
+            segment_algorithms::act_segment(self, action, locator)
         } else {
             // split out the middle
             let mut mid: Treap<D> = self
@@ -54,7 +62,7 @@ impl<D: Data> SomeTree<D> for Treap<D> {
                     Default::default(),
                 ),
             };
-            methods::search_subtree(&mut walker2, locators::RightEdgeOf(locator));
+            walker2.search_subtree(locators::RightEdgeOf(locator));
             let right = walker2.split_right().unwrap();
             drop(walker2);
 
@@ -109,53 +117,16 @@ impl<'a, D: Data> ModifiableTreeRef<D> for &'a mut Treap<D> {
     type ModifiableWalker = TreapWalker<'a, D>;
 }
 
-impl<D: Data> SomeEntry<D> for Treap<D> {
-    fn with_value<F, R>(&mut self, f: F) -> Option<R>
-    where
-        F: FnOnce(&mut D::Value) -> R,
-    {
-        self.tree.with_value(f)
-    }
-
-    fn node_summary(&self) -> D::Summary {
-        self.tree.node_summary()
-    }
-
-    fn subtree_summary(&self) -> D::Summary {
-        self.tree.subtree_summary()
-    }
-
-    fn left_subtree_summary(&self) -> Option<D::Summary> {
-        self.tree.left_subtree_summary()
-    }
-
-    fn right_subtree_summary(&self) -> Option<D::Summary> {
-        self.tree.right_subtree_summary()
-    }
-
-    fn act_subtree(&mut self, action: D::Action) {
-        self.tree.act_subtree(action);
-    }
-
-    fn act_node(&mut self, action: D::Action) -> Option<()> {
-        self.tree.act_node(action)
-    }
-
-    fn act_left_subtree(&mut self, action: D::Action) -> Option<()> {
-        self.tree.act_left_subtree(action)
-    }
-
-    fn act_right_subtree(&mut self, action: D::Action) -> Option<()> {
-        self.tree.act_right_subtree(action)
-    }
-
-    fn assert_correctness_locally(&self)
-    where
-        D::Summary: Eq,
-    {
-        if let Some(node) = self.tree.node() {
-            Self::assert_priorities_locally_internal(node);
-            node.assert_correctness_locally();
+derive_SomeEntry! {tree, T,
+    impl<D: Data> SomeEntry<D> for Treap<D> {
+        fn assert_correctness_locally(&self)
+        where
+            D::Summary: Eq,
+        {
+            if let Some(node) = self.tree.node() {
+                Self::assert_priorities_locally_internal(node);
+                node.assert_correctness_locally();
+            }
         }
     }
 }
@@ -197,7 +168,7 @@ impl<D: Data> Treap<D> {
     /// `O(n*log(n))`.
     pub fn union(&mut self, tree2: Treap<D>)
     where
-        D::Value: Keyed,
+        D::Value: Ord,
     {
         union_internal(&mut self.tree, tree2);
     }
@@ -261,83 +232,24 @@ pub struct TreapWalker<'a, D: Data> {
     walker: BasicWalker<'a, D, T>,
 }
 
-impl<'a, D: Data> SomeWalker<D> for TreapWalker<'a, D> {
-    fn go_left(&mut self) -> Result<(), ()> {
-        self.walker.go_left()
-    }
-
-    fn go_right(&mut self) -> Result<(), ()> {
-        self.walker.go_right()
-    }
-
-    fn go_up(&mut self) -> Result<Side, ()> {
-        self.walker.go_up()
-    }
-
-    fn depth(&self) -> usize {
-        self.walker.depth()
-    }
-
-    fn far_left_summary(&self) -> D::Summary {
-        self.walker.far_left_summary()
-    }
-
-    fn far_right_summary(&self) -> D::Summary {
-        self.walker.far_left_summary()
-    }
-
-    fn value(&self) -> Option<&D::Value> {
-        self.walker.value()
+derive_SomeWalker! {walker,
+    impl<'a, D: Data> SomeWalker<D> for TreapWalker<'a, D> {
+        fn go_up(&mut self) -> Result<Side, ()> {
+            self.walker.go_up()
+        }
     }
 }
 
-impl<'a, D: Data> SomeEntry<D> for TreapWalker<'a, D> {
-    fn with_value<F, R>(&mut self, f: F) -> Option<R>
-    where
-        F: FnOnce(&mut D::Value) -> R,
-    {
-        self.walker.with_value(f)
-    }
-
-    fn node_summary(&self) -> D::Summary {
-        self.walker.node_summary()
-    }
-
-    fn subtree_summary(&self) -> D::Summary {
-        self.walker.subtree_summary()
-    }
-
-    fn left_subtree_summary(&self) -> Option<D::Summary> {
-        self.walker.left_subtree_summary()
-    }
-
-    fn right_subtree_summary(&self) -> Option<D::Summary> {
-        self.walker.right_subtree_summary()
-    }
-
-    fn act_subtree(&mut self, action: D::Action) {
-        self.walker.act_subtree(action);
-    }
-
-    fn act_node(&mut self, action: D::Action) -> Option<()> {
-        self.walker.act_node(action)
-    }
-
-    fn act_left_subtree(&mut self, action: D::Action) -> Option<()> {
-        self.walker.act_left_subtree(action)
-    }
-
-    fn act_right_subtree(&mut self, action: D::Action) -> Option<()> {
-        self.walker.act_right_subtree(action)
-    }
-
-    fn assert_correctness_locally(&self)
-    where
-        D::Summary: Eq,
-    {
-        self.walker.assert_correctness_locally();
-        if let Some(node) = self.walker.node() {
-            Treap::assert_priorities_locally_internal(node);
+derive_SomeEntry! {walker, T,
+    impl<'a, D: Data> SomeEntry<D> for TreapWalker<'a, D> {
+        fn assert_correctness_locally(&self)
+        where
+            D::Summary: Eq,
+        {
+            self.walker.assert_correctness_locally();
+            if let Some(node) = self.walker.node() {
+                Treap::assert_priorities_locally_internal(node);
+            }
         }
     }
 }
@@ -388,8 +300,8 @@ impl<'a, D: Data> ModifiableWalker<D> for TreapWalker<'a, D> {
                     Side::Right => &mut node.right,
                 };
                 std::mem::swap(&mut temp, son);
+                self.walker.rebuild();
             }
-            self.walker.rebuild();
             prev_side = side;
         }
 
@@ -440,7 +352,7 @@ impl<'a, D: Data> ModifiableWalker<D> for TreapWalker<'a, D> {
 /// `O(n*log(n))`.
 fn union_internal<D: Data>(tree1: &mut BasicTree<D, T>, mut tree2: Treap<D>)
 where
-    D::Value: Keyed,
+    D::Value: Ord,
 {
     if tree2.is_empty() {
         return;
@@ -472,18 +384,16 @@ where
 
     let key = node.node_value().get_key(); // this performs access()
 
-    // TODO: replace by a locator that does the handling of the equality case by itself
-    let mut split_walker = methods::search(&mut tree2, locators::ByKey((key,)));
-    // if an element with the same key was found, arbitrarily decide to put it more to the right
-    if !split_walker.is_empty() {
-        methods::previous_empty(&mut split_walker).unwrap();
-    }
+    // if an element with the same key was found, arbitrarily decide to put it more to the left
+    let mut split_walker = tree2.search(
+        locators::LeftEdgeOf(locators::ByKey((key,)))
+    );
+    
     // split
     let right = split_walker.split_right().unwrap();
     drop(split_walker);
     let left = tree2;
 
-    // TODO: nice possible location for parrallelization
     union_internal(&mut node.left, left);
     union_internal(&mut node.right, right);
     node.rebuild();
@@ -498,16 +408,16 @@ where
 /// If elements with equal keys are found, they are placed in an arbitrary order.
 ///
 ///```rust
-///use grove::*;
-///use grove::treap::*;
-///use grove::example_data::{NoAction, Ordered};
+///use grove::{SomeTree, treap, treap::Treap};
+///use grove::example_data::{PlainData};
 ///
-///type T = Treap<NoAction<Ordered<i32>>>;
-///let tree1: T = (0..7).map(|x| Ordered(x)).collect();
-///let tree2: T = (4..9).map(|x| Ordered(x)).collect();
-///let tree = union(tree1, tree2);
+///type T = Treap<PlainData<i32>>;
+///let tree1: T = (0..7).collect();
+///let tree2: T = (4..9).collect();
+///let tree = treap::union(tree1, tree2);
 /// # tree.assert_correctness();
-///assert_eq!(tree.into_iter().collect::<Vec<_>>(), [0,1,2,3,4,4,5,5,6,6,7,8].iter().map(|x| Ordered(*x)).collect::<Vec<_>>());
+///assert_eq!(tree.into_iter().collect::<Vec<_>>(),
+///    [0,1,2,3,4,4,5,5,6,6,7,8].iter().cloned().collect::<Vec<_>>());
 ///```
 ///
 /// # Complexity
@@ -518,7 +428,7 @@ where
 /// `O(n*log(n))`.
 pub fn union<D: Data>(mut tree1: Treap<D>, tree2: Treap<D>) -> Treap<D>
 where
-    D::Value: Keyed,
+    D::Value: Ord,
 {
     tree1.union(tree2);
     tree1
@@ -527,8 +437,7 @@ where
 impl<D: Data> ConcatenableTree<D> for Treap<D> {
     /// Concatenates the trees together, in place.
     ///```
-    /// use grove::trees::*;
-    /// use grove::treap::*;
+    /// use grove::{SomeTree, ConcatenableTree, treap::Treap};
     /// use grove::example_data::StdNum;
     ///
     /// let mut tree: Treap<StdNum> = (17..=89).collect();
@@ -583,10 +492,8 @@ impl<'a, D: Data> SplittableWalker<D> for TreapWalker<'a, D> {
     /// The walker will be at the root after this operation, if it succeeds.
     ///
     ///```
-    /// use grove::trees::*;
-    /// use grove::treap::*;
+    /// use grove::{SomeTree, treap::Treap};
     /// use grove::example_data::StdNum;
-    /// use grove::methods::*;
     ///
     /// let mut tree: Treap<StdNum> = (17..88).collect();
     /// let mut tree2 = tree.slice(7..7).split_right().unwrap();
@@ -630,10 +537,8 @@ impl<'a, D: Data> SplittableWalker<D> for TreapWalker<'a, D> {
     /// The walker will be at the root after this operation, if it succeeds.
     ///
     ///```
-    /// use grove::trees::*;
-    /// use grove::treap::*;
+    /// use grove::{SomeTree, treap::Treap};
     /// use grove::example_data::StdNum;
-    /// use grove::methods::*;
     ///
     /// let mut tree: Treap<StdNum> = (17..88).collect();
     /// let mut tree2 = tree.slice(7..7).split_left().unwrap();
